package com.diagbot.facade;

import com.diagbot.dto.DictionaryInfoDTO;
import com.diagbot.entity.node.*;
import com.diagbot.enums.LabelTypeEnum;
import com.diagbot.exception.CommonErrorCode;
import com.diagbot.exception.CommonException;
import com.diagbot.repository.*;
import com.diagbot.util.BeanUtil;
import com.diagbot.util.Cn2SpellUtil;
import com.diagbot.util.ListUtil;
import com.diagbot.util.StringUtil;
import com.diagbot.vo.EntityInfoVO;
import com.diagbot.vo.EntityPageVO;
import com.google.common.collect.Lists;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.domain.Page;
import org.springframework.data.domain.PageRequest;
import org.springframework.data.domain.Pageable;
import org.springframework.stereotype.Component;

import java.util.List;
import java.util.stream.Collectors;

/**
 * @Description:
 * @Author:zhaops
 * @time: 2020/12/14 16:08
 */
@Component
public class EntityInfoFacade {
    @Autowired
    DictionaryFacade dictionaryFacade;
    @Autowired
    EntityInfoRepository entityInfoRepository;

    @Autowired
    YiBaoDiseaseNameRepository yiBaoDiseaseNameRepository;
    @Autowired
    LisSetRepository lisSetRepository;
    @Autowired
    LisNameRepository lisNameRepository;
    @Autowired
    PacsNameRepository pacsNameRepository;
    @Autowired
    PacsSubNameRepository pacsSubNameRepository;
    @Autowired
    MedicineRepository medicineRepository;
    @Autowired
    YiBaoOperationNameRepository yiBaoOperationNameRepository;
    @Autowired
    DiseaseRepository diseaseRepository;
    @Autowired
    LisRepository lisRepository;
    //TODO 化验细项及结果
    @Autowired
    LisCriticalRepository lisCriticalRepository;
    @Autowired
    LisRemindRepository lisRemindRepository;
    @Autowired
    LisBigNameRepository lisBigNameRepository;
    @Autowired
    PacsCriticalRepository pacsCriticalRepository;
    @Autowired
    PacsDescribeRepository pacsDescribeRepository;
    @Autowired
    PacsResultRepository pacsResultRepository;
    @Autowired
    PacsRemindRepository pacsRemindRepository;
    @Autowired
    MedicineCodeRepository medicineCodeRepository;
    @Autowired
    MedRegNameRepository medRegNameRepository;
    @Autowired
    MedClassRepository medClassRepository;
    @Autowired
    MedZhiLiaoClassRepository medZhiLiaoClassRepository;
    @Autowired
    MedYaoLiClassRepository medYaoLiClassRepository;
    @Autowired
    MedJiePouClassRepository medJiePouClassRepository;
    @Autowired
    MedChemClassRepository medChemClassRepository;
    @Autowired
    OralMedicineRepository oralMedicineRepository;
    @Autowired
    ConflictDeviceRepository conflictDeviceRepository;
    @Autowired
    GenderRepository genderRepository;
    @Autowired
    VitalRepository vitalRepository;
    @Autowired
    SymptomNameRepository symptomNameRepository;
    @Autowired
    ClinicalFindingRepository clinicalFindingRepository;
    @Autowired
    DeptRepository deptRepository;
    @Autowired
    BillConflictItemRepository billConflictItemRepository;
    @Autowired
    GroupRepository groupRepository;
    @Autowired
    TransfusionSuggestRepository transfusionSuggestRepository;
    @Autowired
    MedAllergenRepository medAllergenRepository;
    @Autowired
    AllergenRepository allergenRepository;
    //TODO 食物过敏原
    @Autowired
    TransfusionRemindRepository transfusionRemindRepository;

    /**
     * 分页查询
     *
     * @param entityPageVO
     * @return
     */
    public Page<EntityInfo> getEntityPage(EntityPageVO entityPageVO) {
        List<DictionaryInfoDTO> dicTypeLabelType = dictionaryFacade.getListByGroupType(8);
        List<String> labels = Lists.newArrayList();
        if (StringUtil.isNotBlank(entityPageVO.getLabelType())) {
            labels.add(entityPageVO.getLabelType());
        } else {
            labels = dicTypeLabelType.stream().map(i -> i.getVal()).distinct().collect(Collectors.toList());
        }
        Pageable pageable = PageRequest.of(entityPageVO.getNumber(), entityPageVO.getSize());
        Page<EntityInfo> page = entityInfoRepository.getEntityPage(labels,
                entityPageVO.getName(),
                entityPageVO.getStatus(),
                entityPageVO.getIs_kl(),
                pageable);
        return page;
    }

    /**
     * 唯一键校验
     *
     * @param entityInfoVO
     * @return
     */
    public Boolean isExist(EntityInfoVO entityInfoVO) {
        Boolean exist = false;
        List<EntityInfo> entities = entityInfoRepository.existNodes(entityInfoVO.getName(), entityInfoVO.getLabelType());
        if (ListUtil.isNotEmpty(entities)) {
            if (entityInfoVO.getId() == null) {
                exist = true;
            } else {
                //修改记录
                List<Long> existIds = entities.stream().map(i -> i.getId()).collect(Collectors.toList());
                if (existIds.contains(entityInfoVO.getId())) {
                    exist = false;
                }
            }
        }
        if (!exist) {
            if (entityInfoVO.getLabelType().equals("辅助检查名称")) {
                entities = entityInfoRepository.existNodes(entityInfoVO.getName(), "辅助检查子项目名称");
            } else if (entityInfoVO.getLabelType().equals("辅助检查子项目名称")) {
                entities = entityInfoRepository.existNodes(entityInfoVO.getName(), "辅助检查名称");
            }
            if (ListUtil.isNotEmpty(entities)) {
                if (entityInfoVO.getId() == null) {
                    exist = true;
                } else {
                    //修改记录
                    List<Long> existIds = entities.stream().map(i -> i.getId()).collect(Collectors.toList());
                    if (existIds.contains(entityInfoVO.getId())) {
                        exist = false;
                    }
                }
            }
        }
        return exist;
    }

    public EntityInfo getById(Long id) {
        EntityInfo entityInfo = entityInfoRepository.getById(id);
        if (entityInfo != null) {
            if (entityInfo.getLabelType().equals(LabelTypeEnum.YiBaoDiseaseName.getName())) {
                if (StringUtil.isNotBlank(entityInfo.getAgeRange())) {
                    String[] ages = entityInfo.getAgeRange().split("-");
                    if (ages.length == 2) {
                        entityInfo.setMinAge(Integer.valueOf(ages[0]));
                        entityInfo.setMaxAge(Integer.valueOf(ages[1]));
                    } else if (ages.length == 1) {
                        if (entityInfo.getAgeRange().startsWith("-")) {
                            entityInfo.setMaxAge(Integer.valueOf(ages[0]));
                        } else if (entityInfo.getAgeRange().endsWith("-")) {
                            entityInfo.setMinAge(Integer.valueOf(ages[0]));
                        }
                    }
                }
            }
        }
        return entityInfo;
    }

    /**
     * 术语保存
     *
     * @param entityInfo
     * @return
     */
    public Boolean saveEntity(EntityInfo entityInfo) {
        EntityInfoVO entityInfoVO = new EntityInfoVO();
        BeanUtil.copyProperties(entityInfo, entityInfoVO);
        Boolean exists = isExist(entityInfoVO);
        if (exists) {
            throw new CommonException(CommonErrorCode.IS_EXISTS, "术语已存在");
        }

        //术语类型校验
        List<DictionaryInfoDTO> dicTypeLabelType = dictionaryFacade.getListByGroupType(8);
        if (ListUtil.isEmpty(dicTypeLabelType)) {
            throw new CommonException(CommonErrorCode.PARAM_IS_ERROR, "术语类型字典缺失");
        }
        List<String> labels = dicTypeLabelType.stream().map(i -> i.getVal()).collect(Collectors.toList());
        if (!labels.contains(entityInfo.getLabelType())) {
            throw new CommonException(CommonErrorCode.PARAM_IS_ERROR, "术语类型错误");
        }

        //默认值处理
        if (StringUtil.isBlank(entityInfo.getPycode())) {
            entityInfo.setPycode(Cn2SpellUtil.converterToFirstSpell(entityInfo.getName()));
        }
        if (entityInfo.getStatus() == null) {
            entityInfo.setStatus(1);
        }
        if (entityInfo.getIs_kl() == null) {
            entityInfo.setIs_kl(0);
        }

        switch (entityInfo.getLabelType()) {
            case "医保疾病名称":
                entityInfo.setAgeRange(entityInfo.getMinAge() + "-" + entityInfo.getMaxAge());
                if (entityInfo.getId() != null) {
                    yiBaoDiseaseNameRepository.update(entityInfo,
                            entityInfo.getAgeRange(),
                            entityInfo.getAlias(),
                            entityInfo.getCategory(),
                            entityInfo.getDept(),
                            entityInfo.getGender());
                } else {
                    yiBaoDiseaseNameRepository.create(entityInfo,
                            entityInfo.getAgeRange(),
                            entityInfo.getAlias(),
                            entityInfo.getCategory(),
                            entityInfo.getDept(),
                            entityInfo.getGender());
                }
                break;
            case "实验室检查名称":
                if (entityInfo.getId() == null) {
                    lisNameRepository.create(entityInfo, entityInfo.getUnit(), entityInfo.getMaxval(), entityInfo.getMinval(), entityInfo.getRange());
                } else {
                    lisNameRepository.update(entityInfo, entityInfo.getUnit(), entityInfo.getMaxval(), entityInfo.getMinval(), entityInfo.getRange());
                }
                break;
            case "辅助检查名称":
                if (entityInfo.getId() != null) {
                    pacsNameRepository.update(entityInfo, entityInfo.getGender());
                } else {
                    pacsNameRepository.create(entityInfo, entityInfo.getGender());
                }
                break;
            case "辅助检查子项目名称":
                if (entityInfo.getId() != null) {
                    pacsSubNameRepository.update(entityInfo, entityInfo.getGender());
                } else {
                    pacsSubNameRepository.create(entityInfo, entityInfo.getGender());
                }
                break;
            case "医保手术和操作名称":
                if (entityInfo.getOpgrade() == null) {
                    entityInfo.setOpgrade("0");
                }
                if (entityInfo.getId() != null) {
                    yiBaoOperationNameRepository.update(entityInfo, entityInfo.getOpgrade());
                } else {
                    yiBaoOperationNameRepository.create(entityInfo, entityInfo.getOpgrade());
                }
                break;
            case "实验室检查":
                if (entityInfo.getId() == null) {
                    lisRepository.create(entityInfo, entityInfo.getUnit());
                } else {
                    lisRepository.update(entityInfo, entityInfo.getUnit());
                }
                break;
            case "化验细项及结果":
                //特殊处理
                break;
            case "实验室检查危急值":
                if (entityInfo.getId() == null) {
                    lisCriticalRepository.create(entityInfo, entityInfo.getUnit());
                } else {
                    lisCriticalRepository.update(entityInfo, entityInfo.getUnit());
                }
                break;
            case "药品代码通用名":
                //特殊处理
                break;
            case "药品注册名称":
                //特殊处理
                break;
            case "体征":
                if (entityInfo.getId() == null) {
                    vitalRepository.create(entityInfo, entityInfo.getUnit());
                } else {
                    vitalRepository.update(entityInfo, entityInfo.getUnit());
                }
                break;
            case "药物过敏原":
                if (StringUtil.isBlank(entityInfo.getMedtype())) {
                    throw new CommonException(CommonErrorCode.PARAM_IS_NULL, "请输入药品类型");
                }
                if (entityInfo.getId() == null) {
                    medAllergenRepository.create(entityInfo, entityInfo.getMedtype(), entityInfo.getResult(), entityInfo.getAlias());
                } else {
                    medAllergenRepository.update(entityInfo, entityInfo.getMedtype(), entityInfo.getResult(), entityInfo.getAlias());
                }
                break;
            case "实验室检查套餐名":
                if (entityInfo.getId() == null) {
                    LisSet lisSet = new LisSet();
                    BeanUtil.copyProperties(entityInfo, lisSet);
                    lisSetRepository.save(lisSet);
                } else {
                    entityInfoRepository.update(entityInfo);
                }
                break;
            case "药品通用名称":
                if (entityInfo.getId() == null) {
                    Medicine medicine = new Medicine();
                    BeanUtil.copyProperties(entityInfo, medicine);
                    medicineRepository.save(medicine);
                } else {
                    entityInfoRepository.update(entityInfo);
                }
                break;
            case "疾病":
                if (entityInfo.getId() == null) {
                    Disease disease = new Disease();
                    BeanUtil.copyProperties(entityInfo, disease);
                    diseaseRepository.save(disease);
                } else {
                    entityInfoRepository.update(entityInfo);
                }
                break;
            case "化验提醒指标":
                if (entityInfo.getId() == null) {
                    LisRemind lisRemind = new LisRemind();
                    BeanUtil.copyProperties(entityInfo, lisRemind);
                    lisRemindRepository.save(lisRemind);
                } else {
                    entityInfoRepository.update(entityInfo);
                }
                break;
            case "化验套餐名称":
                if (entityInfo.getId() == null) {
                    LisBigName lisBigName = new LisBigName();
                    BeanUtil.copyProperties(entityInfo, lisBigName);
                    lisBigNameRepository.save(lisBigName);
                } else {
                    entityInfoRepository.update(entityInfo);
                }
                break;
            case "辅助检查危急值":
                if (entityInfo.getId() == null) {
                    PacsCritical pacsCritical = new PacsCritical();
                    BeanUtil.copyProperties(entityInfo, pacsCritical);
                    pacsCriticalRepository.save(pacsCritical);
                } else {
                    entityInfoRepository.update(entityInfo);
                }
                break;
            case "辅助检查名称描述":
                if (entityInfo.getId() == null) {
                    PacsDescribe pacsDescribe = new PacsDescribe();
                    BeanUtil.copyProperties(entityInfo, pacsDescribe);
                    pacsDescribeRepository.save(pacsDescribe);
                } else {
                    entityInfoRepository.update(entityInfo);
                }
                break;
            case "辅助检查名称结果":
                if (entityInfo.getId() == null) {
                    PacsResult pacsResult = new PacsResult();
                    BeanUtil.copyProperties(entityInfo, pacsResult);
                    pacsResultRepository.save(pacsResult);
                } else {
                    entityInfoRepository.update(entityInfo);
                }
                break;
            case "辅检提醒指标":
                if (entityInfo.getId() == null) {
                    PacsRemind pacsRemind = new PacsRemind();
                    BeanUtil.copyProperties(entityInfo, pacsRemind);
                    pacsRemindRepository.save(pacsRemind);
                } else {
                    entityInfoRepository.update(entityInfo);
                }
                break;
            case "药品类别":
                if (entityInfo.getId() == null) {
                    MedClass medClass = new MedClass();
                    BeanUtil.copyProperties(entityInfo, medClass);
                    medClassRepository.save(medClass);
                } else {
                    entityInfoRepository.update(entityInfo);
                }
                break;
            case "药品治疗学类别":
                if (entityInfo.getId() == null) {
                    MedZhiLiaoClass medZhiLiaoClass = new MedZhiLiaoClass();
                    BeanUtil.copyProperties(entityInfo, medZhiLiaoClass);
                    medZhiLiaoClassRepository.save(medZhiLiaoClass);
                } else {
                    entityInfoRepository.update(entityInfo);
                }
                break;
            case "药品药理学类别":
                if (entityInfo.getId() == null) {
                    MedYaoLiClass medYaoLiClass = new MedYaoLiClass();
                    BeanUtil.copyProperties(entityInfo, medYaoLiClass);
                    medYaoLiClassRepository.save(medYaoLiClass);
                } else {
                    entityInfoRepository.update(entityInfo);
                }
                break;
            case "药品解剖学类别":
                if (entityInfo.getId() == null) {
                    MedJiePouClass medJiePouClass = new MedJiePouClass();
                    BeanUtil.copyProperties(entityInfo, medJiePouClass);
                    medJiePouClassRepository.save(medJiePouClass);
                } else {
                    entityInfoRepository.update(entityInfo);
                }
                break;
            case "药品化学物质类别":
                if (entityInfo.getId() == null) {
                    MedChemClass medChemClass = new MedChemClass();
                    BeanUtil.copyProperties(entityInfo, medChemClass);
                    medChemClassRepository.save(medChemClass);
                } else {
                    entityInfoRepository.update(entityInfo);
                }
                break;
            case "服用药品":
                if (entityInfo.getId() == null) {
                    OralMedicine oralMedicine = new OralMedicine();
                    BeanUtil.copyProperties(entityInfo, oralMedicine);
                    oralMedicineRepository.save(oralMedicine);
                } else {
                    entityInfoRepository.update(entityInfo);
                }
                break;
            case "禁忌医疗器械及物品":
                if (entityInfo.getId() == null) {
                    ConflictDevice conflictDevice = new ConflictDevice();
                    BeanUtil.copyProperties(entityInfo, conflictDevice);
                    conflictDeviceRepository.save(conflictDevice);
                } else {
                    entityInfoRepository.update(entityInfo);
                }
                break;
            case "性别":
                if (entityInfo.getId() == null) {
                    Gender gender = new Gender();
                    BeanUtil.copyProperties(entityInfo, gender);
                    genderRepository.save(gender);
                } else {
                    entityInfoRepository.update(entityInfo);
                }
                break;
            case "症状":
                if (entityInfo.getId() == null) {
                    Symptom symptom = new Symptom();
                    BeanUtil.copyProperties(entityInfo, symptom);
                    symptomNameRepository.save(symptom);
                } else {
                    entityInfoRepository.update(entityInfo);
                }
                break;
            case "临床表现":
                if (entityInfo.getId() == null) {
                    ClinicalFinding clinicalFinding = new ClinicalFinding();
                    BeanUtil.copyProperties(entityInfo, clinicalFinding);
                    clinicalFindingRepository.save(clinicalFinding);
                } else {
                    entityInfoRepository.update(entityInfo);
                }
                break;
            case "科室":
                if (entityInfo.getId() == null) {
                    Dept dept = new Dept();
                    BeanUtil.copyProperties(entityInfo, dept);
                    deptRepository.save(dept);
                } else {
                    entityInfoRepository.update(entityInfo);
                }
                break;
            case "开单项互斥":
                if (entityInfo.getId() == null) {
                    BillConflictItem billConflictItem = new BillConflictItem();
                    BeanUtil.copyProperties(entityInfo, billConflictItem);
                    billConflictItemRepository.save(billConflictItem);
                } else {
                    entityInfoRepository.update(entityInfo);
                }
                break;
            case "禁忌人群":
                if (entityInfo.getId() == null) {
                    Group group = new Group();
                    BeanUtil.copyProperties(entityInfo, group);
                    groupRepository.save(group);
                } else {
                    entityInfoRepository.update(entityInfo);
                }
                break;
            case "建议输血提醒":
                if (entityInfo.getId() == null) {
                    TransfusionSuggest transfusionSuggest = new TransfusionSuggest();
                    BeanUtil.copyProperties(entityInfo, transfusionSuggest);
                    transfusionSuggestRepository.save(transfusionSuggest);
                } else {
                    entityInfoRepository.update(entityInfo);
                }
                break;
            case "过敏原":
                if (entityInfo.getId() == null) {
                    Allergen allergen = new Allergen();
                    BeanUtil.copyProperties(entityInfo, allergen);
                    allergenRepository.save(allergen);
                } else {
                    entityInfoRepository.update(entityInfo);
                }
                break;
            case "食物过敏原":
                break;
            case "输血提醒指标":
                if (entityInfo.getId() == null) {
                    TransfusionRemind transfusionRemind = new TransfusionRemind();
                    BeanUtil.copyProperties(entityInfo, transfusionRemind);
                    transfusionRemindRepository.save(transfusionRemind);
                } else {
                    entityInfoRepository.update(entityInfo);
                }
                break;
            default:
                break;
        }
        return true;
    }
}