package com.diagbot.web;

import com.diagbot.dto.RespDTO;
import com.diagbot.entity.node.EntityInfo;
import com.diagbot.facade.EntityInfoFacade;
import com.diagbot.vo.EntityInfoVO;
import com.diagbot.vo.EntityPageVO;
import com.diagbot.vo.IdVO;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.domain.Page;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

/**
 * @Description:
 * @Author:zhaops
 * @time: 2020/12/14 16:15
 */
@RestController
@RequestMapping("/entityman")
@Api(value = "知识图谱标准术语维护相关API", tags = { "知识图谱标准术语维护相关API" })
@SuppressWarnings("unchecked")
public class EntityInfoController {
    @Autowired
    EntityInfoFacade entityInfoFacade;

    @ApiOperation(value = "分页查询[zhaops]",
            notes = "number:页码<br>" +
                    "size:每页条目数<br>" +
                    "name:术语名称（模糊匹配）<br>" +
                    "labelType:术语类型-知识图谱标签类型（String）<br>" +
                    "is_kl:静态知识标志（0-无，1-有）<br>" +
                    "status:启用禁用标志（0-禁用，1-启用）<br>")
    @PostMapping("/getEntityPage")
    public RespDTO<EntityInfo> getEntityPage(@RequestBody EntityPageVO entityPageVO) {
        Page<EntityInfo> data = entityInfoFacade.getEntityPage(entityPageVO);
        return RespDTO.onSuc(data);
    }

    @ApiOperation(value = "唯一键校验[zhaops]",
            notes = "name:术语名称（模糊匹配）<br>" +
                    "labelType:术语类型-知识图谱标签类型（String）<br>")
    @PostMapping("/isExist")
    public RespDTO<Boolean> isExist(@RequestBody EntityInfoVO entityInfoVO) {
        Boolean data = entityInfoFacade.isExist(entityInfoVO);
        return RespDTO.onSuc(data);
    }

    @ApiOperation(value = "保存[zhaops]",
            notes = "id:术语ID<br>" +
                    "name:术语名称<br>" +
                    "pycode:拼音编码<br>" +
                    "status:状态（0-禁用，1-启用，默认1）<br>" +
                    "is_kl:静态知识标识（0-无，1-有，默认0）<br>" +
                    "labelType:术语类型-知识图谱标签类型（详情见字典）<br>" +
                    "age:年龄<br>" +
                    "gender:性别<br>" +
                    "unit:单位（实验室检查名称、实验室检查、体征、实验室检查危急值）<br>" +
                    "alias:别名（医保诊断名称、药物过敏原、药品注册名称）<br>" +
                    "result:结果（药物过敏原、药品注册名称）<br>" +
                    "minval:最小值（实验室检查名称）<br>" +
                    "maxval:最大值（实验室检查名称）<br>" +
                    "range:范围（0-范围内，1-范围外，默认0）<br>" +
                    "category:诊断分类（医保诊断名称属性：1、2，无默认值）<br>" +
                    "dept:诊断所属科室（医保诊断名称属性）<br>" +
                    "minAge:年龄下限（医保诊断名称属性）<br>" +
                    "maxAge:年龄上限（医保诊断名称属性）<br>" +
                    "opgrade:手术等级（医保手术和操作属性：0、1、2、3、4，默认0）<br>" +
                    "highriskcond:高危条件（医保手术和操作属性）<br>" +
                    "special:特殊标志（化验细项及结果属性：是）<br>" +
                    "medtype:药品类型（药物过敏原属性：药品、药品类别）<br>" +
                    "risklevel:药品高危级别（药品注册名称属性）<br>" +
                    "conflict_geiyao:禁忌给药途径（药品注册名称属性）<br>" +
                    "conflict_gender:禁忌性别（药品注册名称属性）<br>")
    @PostMapping("/saveEntity")
    public RespDTO<Boolean> saveEntity(@RequestBody EntityInfo entityInfo) {
        Boolean data = entityInfoFacade.saveEntity(entityInfo);
        return RespDTO.onSuc(data);
    }

    @ApiOperation(value = "查看实体对象[zhaops]",
            notes = "id:实体id<br>")
    @PostMapping("/getById")
    public RespDTO<EntityInfo> getById(@RequestBody IdVO idVO) {
        EntityInfo data = entityInfoFacade.getById(idVO.getId());
        return RespDTO.onSuc(data);
    }
}